@@ -1291,16 +1291,16 @@ gvm_device_mounted (const char *udi)
  * @return TRUE iff the mount was succesful
  */
 static gboolean
-gvm_device_mount (const char *udi, int apply_policy)
+gvm_device_mount (const char *udi, gboolean interactive)
 {
-	char *mount_point, *fstype, *options[2], **poptions;
+	char *command, *mount_point, *fstype;
 	struct _MountPolicy *policy;
 	DBusMessage *dmesg, *reply;
 	dbus_uint64_t size;
-	int noptions = 0;
+	GPtrArray *options;
 	DBusError error;
 	gboolean retval;
-	char *command;
+	char uid[32];
 	
 	dbg ("mounting %s...\n", udi);
 	
@@ -1312,11 +1312,16 @@ gvm_device_mount (const char *udi, int apply_policy)
 	if (gnome_mount != NULL) {
 		policy = g_new (struct _MountPolicy, 1);
 		policy->udi = g_strdup (udi);
-		policy->apply = apply_policy;
+		policy->apply = interactive;
 		
 		g_hash_table_insert (mount_table, policy->udi, policy);
 		
-		command = g_strdup_printf ("%s --hal-udi=%%h", gnome_mount);
+		if (!interactive) {
+			command = g_strdup_printf ("%s --no-ui --hal-udi=%%h", gnome_mount);
+		} else {
+			command = g_strdup_printf ("%s --hal-udi=%%h", gnome_mount);
+		}
+		
 		retval = gvm_run_command (command, udi, NULL, NULL);
 		g_free (command);
 		
@@ -1336,26 +1341,41 @@ gvm_device_mount (const char *udi, int apply_policy)
 		}
 		
 		size = libhal_device_get_property_uint64 (hal_ctx, udi, "volume.size", NULL);
+		if (!(fstype = libhal_device_get_property_string (hal_ctx, udi, "volume.fstype", NULL)))
+			fstype = g_strdup ("");
 		
-		fstype = "";
 		mount_point = "";
 		
-		if (size <= (256 * 1024 * 1024))
-			options[noptions++] = "sync";
-		options[noptions] = NULL;
-		poptions = options;
+		snprintf (uid, sizeof (uid) - 1, "uid=%u", getuid ());
+		
+		options = g_ptr_array_new ();
+		
+		if (!strcmp (fstype, "vfat")) {
+			g_ptr_array_add (options, "shortname=winnt");
+			g_ptr_array_add (options, uid);
+			
+			if (size <= (512 * 1024 * 1024))
+				g_ptr_array_add (options, "sync");
+		} else if (!strcmp (fstype, "iso9660")) {
+			g_ptr_array_add (options, uid);
+		} else if (!strcmp (fstype, "udf")) {
+			g_ptr_array_add (options, uid);
+		}
 		
 		if (!dbus_message_append_args (dmesg, DBUS_TYPE_STRING, &mount_point, DBUS_TYPE_STRING, &fstype,
-					       DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &poptions, noptions,
+					       DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &options->pdata, options->len,
 					       DBUS_TYPE_INVALID)) {
 			dbg ("mount failed for %s: could not append args to dbus message\n", udi);
 			dbus_message_unref (dmesg);
 			return FALSE;
 		}
 		
+		g_ptr_array_free (options, TRUE);
+		g_free (fstype);
+		
 		policy = g_new (struct _MountPolicy, 1);
 		policy->udi = g_strdup (udi);
-		policy->apply = apply_policy;
+		policy->apply = interactive;
 		
 		g_hash_table_insert (mount_table, policy->udi, policy);
 		
@@ -2340,6 +2360,7 @@ mount_all (LibHalContext *ctx)
 		/* mount the device */
 		if ((dev = libhal_device_get_property_string (ctx, udi, "block.device", &error))) {
 			dbg ("mount_all: mounting %s\n", dev);
+			/* don't make the mount program put up error dialogs */
 			gvm_device_mount (udi, FALSE);
 			libhal_free_string (dev);
 		} else {
@@ -2453,9 +2474,25 @@ gvm_user_active_at_console (void)
 }
 
 
-#ifndef POPT_TABLEEND
-#define POPT_TABLEEND { NULL, '\0', 0, NULL, '\0', NULL, NULL }
-#endif
+static gboolean print_version;
+static const char *daemon_arg;
+static gboolean no_daemon;
+static gboolean secret_mode;
+
+#ifdef GNOME_PARAM_GOPTION_CONTEXT
+static GOptionEntry options[] = {
+	{ "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &print_version,
+	  N_("Print version and exit"), NULL },
+	{ "daemon", 'd', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_STRING, (char **) &daemon_arg,
+	  N_("Run as a daemon"), "<yes|no>" },
+	{ "no-daemon", 'n', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &no_daemon,
+	  N_("Don't run as a daemon"), NULL },
+	{ "secret-mode", 's', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &secret_mode,
+	  N_("Run in secret mode"), NULL },
+	{ NULL, '\0', 0, 0, NULL, NULL, NULL}
+};
+
+#else
 
 static const struct poptOption options[] = {
 	{ "version", 'v', POPT_ARG_NONE | POPT_ARGFLAG_ONEDASH, NULL, 'v',
@@ -2466,8 +2503,9 @@ static const struct poptOption options[] = {
 	  N_("Don't run as a daemon"), NULL },
 	{ "secret-mode", 's', POPT_ARG_NONE | POPT_ARGFLAG_ONEDASH, NULL, 's',
 	  N_("Run in secret mode"), NULL },
-	POPT_TABLEEND
+	{ NULL, '\0', 0, NULL, '\0', NULL, NULL }
 };
+#endif
 
 int
 main (int argc, char *argv[])
@@ -2475,41 +2513,73 @@ main (int argc, char *argv[])
 	gboolean daemonize = TRUE;
 	GnomeProgram *program;
 	GnomeClient *client;
+#ifdef GNOME_PARAM_GOPTION_CONTEXT
+	GOptionContext *ctx;
+#else
 	poptContext ctx;
-	const char *arg;
 	int rc;
+#endif
 	
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (PACKAGE, "UTF-8");
 	textdomain (PACKAGE);
 	
+#ifdef GNOME_PARAM_GOPTION_CONTEXT
+	context = g_option_context_new (PACKAGE);
+	g_option_context_add_main_entries (context, options, NULL);
+#endif
+	
 	program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
+#ifdef GNOME_PARAM_GOPTION_CONTEXT
+				      GNOME_PARAM_GOPTION_CONTEXT, ctx,
+#else
 				      GNOME_PARAM_POPT_FLAGS, POPT_CONTEXT_POSIXMEHARDER,
-				      GNOME_PARAM_POPT_TABLE, options, GNOME_PARAM_NONE);
+				      GNOME_PARAM_POPT_TABLE, options,
+#endif
+				      GNOME_PARAM_NONE);
 	
+#ifndef GNOME_PARAM_GOPTION_CONTEXT
 	g_object_get ((GObject *) program, GNOME_PARAM_POPT_CONTEXT, &ctx, NULL);
-	
+
 	poptResetContext (ctx);
 	
 	while ((rc = poptGetNextOpt (ctx)) > 0) {
 		switch (rc) {
 		case 'v':
-			fprintf (stdout, "%s version %s\n", PACKAGE, VERSION);
-			poptFreeContext (ctx);
-			exit (0);
+			print_version = TRUE;
+			break;
 		case 'd':
-			if (!(arg = poptGetOptArg (ctx)) || (!strcmp (arg, "true") || !strcmp (arg, "yes"))) {
-				daemonize = TRUE;
-				break;
-			}
+			daemon_arg = poptGetOptArg (ctx);
+			break;
 		case 'n':
-			daemonize = FALSE;
+			no_daemon = TRUE;
 			break;
 		case 's':
-			fprintf (stdout, "Managing volumes since the summer of '89\n");
+			secret_mode = TRUE;
 			break;
 		}
 	}
+#endif
+	
+	if (print_version) {
+		fprintf (stdout, "%s version %s\n", PACKAGE, VERSION);
+		exit (0);
+	}
+	
+	if (daemon_arg != NULL) {
+		if (!strcmp (daemon_arg, "yes") || !strcmp (daemon_arg, "true"))
+			daemonize = TRUE;
+		else if (!strcmp (daemon_arg, "no") || !strcmp (daemon_arg, "false"))
+			daemonize = FALSE;
+		else
+			fprintf (stdout, _("Unrecogonized --daemon argument: %s\n"), daemon_arg);
+	}
+	
+	if (no_daemon)
+		daemonize = FALSE;
+	
+	if (secret_mode)
+		fprintf (stdout, "Managing volumes since the summer of '89\n");
 	
 	if (daemonize && daemon (0, 0) < 0) {
 		warn ("daemonizing failed: %s", g_strerror (errno));
@@ -2556,8 +2626,8 @@ main (int argc, char *argv[])
 	
 	gtk_main ();
 	
+	g_hash_table_destroy (mount_table);
 	g_object_unref (program);
-	poptFreeContext (ctx);
 	
 	return 0;
 }
